require "rails/generators"
require "redis/objects/sorted_sets"

module Patriarch
  module Generators
    class BehaviourGenerator < Rails::Generators::Base
      source_root File.expand_path('../templates', __FILE__)
      
      desc "Generate files needed to implement the BEHAVIOUR you specified. Don't forget to add declarations into models"
    
      argument :behaviour, :type => :string
      argument :behaviour_type, :type => :string, :optional => true, :default => "bipartite"

      public

      def fail_if_bad_syntax
        if behaviour_type != "bipartite" && behaviour_type != "tripartite"
          # Pleasedon't, qualify an exception here
          raise Exception, "bad syntax behaviour_type must be bipartite or tripartite"
        end 
      end

      def init_directories_for_behaviour
        empty_directory "lib/patriarch/services/#{behaviour.underscore.downcase}"
        empty_directory "lib/patriarch/services/#{Patriarch.undo(behaviour.underscore.downcase)}"
      end

      def customize_basic_files 
        template "empty_behaviour_module.rb", "lib/patriarch/behaviours/#{behaviour.underscore.downcase}.rb"   
        template "empty_service_module.rb", "lib/patriarch/services/#{behaviour.underscore.downcase}.rb"         
        template "tools_methods.rb", "lib/patriarch/behaviours/#{behaviour.underscore.downcase}/tools_methods.rb"
      end
    
      def ensure_autoload
        # Autoload declaration insertion for services
        insert_into_file "lib/patriarch/services.rb", :after => "  module Services\n" do
          "    autoload :#{class_name}, 'patriarch/services/#{behaviour.underscore.downcase}.rb'\n"
        end
    
        insert_into_file "lib/patriarch/services.rb", :after => "  module Services\n" do
          "    autoload :#{undo_class_name}, 'patriarch/services/#{behaviour.underscore.downcase}.rb'\n"
        end
    
        # Autoload declaration insertion for behaviours (no undo needed ...)
        insert_into_file "lib/patriarch/behaviours.rb", :after => "  module Behaviours\n" do
          "    autoload :#{class_name}, 'patriarch/behaviours/#{behaviour.underscore.downcase}.rb'\n"
        end
    
        # load File.expand_path('lib/patriarch/behaviours.rb', __FILE__)
      end
    
      def generate_services
        create_services(behaviour,behaviour_type)
        # implémenter un switch ici, plus zoli ...
        self.class.send(:define_method,:class_name) do
          Patriarch.undo(behaviour).classify
        end
        create_undo_services(behaviour,behaviour_type)
        self.class.send(:define_method,:class_name) do
          behaviour.classify
        end
      end
    
      private
        def class_name
          behaviour.classify
        end
    
        def undo_class_name
          Patriarch.undo(behaviour).classify
        end
    
        def create_services(behaviour,behaviour_type)
          behaviour_str = behaviour.underscore.downcase
          template "authorization_service.rb", "lib/patriarch/services/#{behaviour_str}/authorization_service.rb"
          
          template "before_manager_service.rb", "lib/patriarch/services/#{behaviour_str}/before_manager_service.rb"
          template "before_service.rb", "lib/patriarch/services/#{behaviour_str}/before_service.rb"
    

          
          if behaviour_type == "tripartite"  
            template "service-tripartite.rb", "lib/patriarch/services/#{behaviour_str}/service.rb"  
            template "manager_service-tripartite.rb", "lib/patriarch/services/#{behaviour_str}/manager_service.rb"     
          elsif behaviour_type == "bipartite"          
            template "service.rb", "lib/patriarch/services/#{behaviour_str}/service.rb"  
            template "manager_service.rb", "lib/patriarch/services/#{behaviour_str}/manager_service.rb"            
          end

          template "after_manager_service.rb", "lib/patriarch/services/#{behaviour_str}/after_manager_service.rb"
          template "after_service.rb", "lib/patriarch/services/#{behaviour_str}/after_service.rb"
        end
    
        def create_undo_services(behaviour,behaviour_type)
          undo_behaviour_str = Patriarch.undo(behaviour).underscore.downcase
    
          template "authorization_service.rb", "lib/patriarch/services/#{undo_behaviour_str}/authorization_service.rb"
          
          template "before_manager_service.rb", "lib/patriarch/services/#{undo_behaviour_str}/before_manager_service.rb"
          template "before_service.rb", "lib/patriarch/services/#{undo_behaviour_str}/before_service.rb"
          
          # Please don't go another generator if necessary or do other functions
          if behaviour_type == "tripartite"                      
            template "undo_service-tripartite.rb", "lib/patriarch/services/#{undo_behaviour_str}/service.rb" 
            template "manager_service-tripartite.rb", "lib/patriarch/services/#{undo_behaviour_str}/manager_service.rb"               
          elsif behaviour_type == "bipartite"
            template "undo_service.rb", "lib/patriarch/services/#{undo_behaviour_str}/service.rb"  
            template "manager_service.rb", "lib/patriarch/services/#{undo_behaviour_str}/manager_service.rb"            
          end

          template "after_manager_service.rb", "lib/patriarch/services/#{undo_behaviour_str}/after_manager_service.rb"
          template "after_service.rb", "lib/patriarch/services/#{undo_behaviour_str}/after_service.rb"
        end
    
    end # BehaviourGenerator
  end # Generators
end # Patriarch
